from django.core.management.base import BaseCommand
from django.db.models.loading import get_model
from django.conf import settings

from sampledatahelper.model_helper import ModelDataHelper

class Command(BaseCommand):
    args = ''
    help = 'Example data generator'
    mdh = ModelDataHelper(seed=12345678901)

    def handle(self, *args, **options):
        for model_conf in settings.SAMPLEDATAHELPER_MODELS:
            app_label, model_name = model_conf.get('model').split(".")
            instances_number = model_conf.get('number')
            fields_overwrite = model_conf.get('fields_overwrite', {})
            model = get_model(app_label, model_name)
            self.mdh.fill_model(model, instances_number, *fields_overwrite)



import json
import os.path

from sampledatahelper.exceptions import ParameterError

LOCALES = ['es', 'us', 'cat', 'fr']
NAMES_PATH = os.path.join(os.path.dirname(__file__), 'names')


class Name(object):
    data = {}

    def __load_locale(self, locale):
        locale_path = os.path.join(NAMES_PATH, "{0}.json".format(locale))
        if not os.path.exists(locale_path):
            raise ParameterError('Not valid locale')

        fd = open(locale_path, 'r')
        Name.data[locale] = json.load(fd)
        fd.close()

    def get_names(self, locale):
        if locale not in Name.data:
            self.__load_locale(locale)

        return [x[0] for x in Name.data[locale]['names']]

    def get_names_number(self, locale):
        if locale not in Name.data:
            self.__load_locale(locale)

        return Name.data[locale]['names_number']

    def all_names(self):
        names = []
        for locale in LOCALES:
            names += self.get_names(locale)

        return names

    def generate(self, sd, locale=None, number=None, as_list=False):
        if locale:
            names = self.get_names(locale)
            if number is None:
                number = self.get_names_number(locale)
        else:
            names = self.all_names()

        if number is None:
            number = 1

        if number < 1:
            raise ParameterError("number must be greater than 1")

        result = []
        for x in range(number):
            result.append(sd.choice(names))

        if as_list:
            return result

        return ' '.join(result)


class Surname(object):
    data = {}

    def __load_locale(self, locale):
        locale_path = os.path.join(NAMES_PATH, "{0}.json".format(locale))
        if not os.path.exists(locale_path):
            raise ParameterError('Not valid locale')

        fd = open(locale_path, 'r')
        Surname.data[locale] = json.load(fd)
        fd.close()

    def get_surnames(self, locale):
        if locale not in Surname.data:
            self.__load_locale(locale)

        return [x[0] for x in Surname.data[locale]['surnames']]

    def get_surnames_number(self, locale):
        if locale not in Surname.data:
            self.__load_locale(locale)

        return Surname.data[locale]['surnames_number']

    def all_surnames(self):
        surnames = []
        for locale in LOCALES:
            surnames += self.get_surnames(locale)

        return surnames

    def generate(self, sd, locale=None, number=None, as_list=False):
        if locale:
            surnames = self.get_surnames(locale)
            if number is None:
                number = self.get_surnames_number(locale)
        else:
            surnames = self.all_surnames()

        result = []

        if number is None:
            number = 1

        if number < 1:
            raise ParameterError("number must be greater than 1")

        for x in range(number):
            result.append(sd.choice(surnames))

        if as_list:
            return result

        return ' '.join(result)


class FullName(object):
    def generate(self, sd, locale=None, as_list=False):
        sngen = Surname()
        ngen = Name()

        names_number = 1
        surnames_number = 1

        if locale:
            names_number = ngen.get_names_number(locale)
            surnames_number = sngen.get_surnames_number(locale)

        names = ngen.generate(sd, locale, names_number, True)
        surnames = sngen.generate(sd, locale, surnames_number, True)

        if as_list:
            return names + surnames

        return ' '.join(names + surnames)



import os
import io
import random
from django.core.files.images import ImageFile
try:
    from sampledatahelper.image_generators import ImgSimple, ImgPlasma, ImgMandelbrot, ImgIFS
    PIL_INSTALLED = True
except ImportError:
    PIL_INSTALLED = False
from tempfile import NamedTemporaryFile

from sampledatahelper.exceptions import ParameterError, NotChoicesFound


class ImageMixin(object):
    def image_from_directory(self, directory_path, valid_extensions=['.jpg', '.bmp', '.png']):
        if not os.path.exists(directory_path):
            raise ParameterError('directory_path must be a valid path')

        list_of_images = os.listdir(directory_path)
        list_of_images = list(filter(lambda x: os.path.splitext(x)[1] in valid_extensions, list_of_images))

        if len(list_of_images) == 0:
            raise NotChoicesFound('Not valid images found in directory_path for valid_extensions')

        random_path = os.path.join(directory_path, random.choice(list_of_images))

        fd = open(random_path, 'rb')
        stream = io.BytesIO(fd.read())
        fd.close()
        im_file = ImageFile(stream)
        return im_file

    def image(self, width, height, typ="simple"):
        if not PIL_INSTALLED:
            raise ImportError("PIL needed to use this function")

        if width <= 0 or height <= 0:
            raise ParameterError('width and width must be geater than 0')

        if typ == "simple":
            generator = ImgSimple()
        elif typ == "plasma":
            generator = ImgPlasma()
        elif typ == "mandelbrot":
            generator = ImgMandelbrot()
        elif typ == "ifs":
            generator = ImgIFS()
        elif typ == "random":
            generator = self.choice([ImgSimple, ImgPlasma, ImgMandelbrot, ImgIFS])()
        else:
            raise ParameterError('Unknown image generator type')

        im = generator.generate(self, width, height)

        tf = NamedTemporaryFile(delete=False, suffix=".png")
        tf.close()

        im.save(tf.name)

        fd = open(tf.name, 'rb')
        stream = io.BytesIO(fd.read())
        fd.close()
        im_file = ImageFile(stream)
        os.unlink(tf.name)
        return im_file



import datetime as dt
import random
from django.utils.timezone import utc

from ..exceptions import ParameterError


class TimeMixin(object):
    def date_between(self, min_date, max_date):
        """Random date between a min_date and a max_date."""
        if min_date > max_date:
            raise ParameterError('min_date greater than max_date')

        delta = max_date - min_date
        int_delta = delta.days
        random_day = self.int(0, int_delta)

        return min_date + dt.timedelta(days=random_day)

    def future_date(self, min_distance=0, max_distance=365):
        """Random date between today and today + one year - one day."""
        if min_distance < 0:
            raise ParameterError('min_distance less than 0')

        if min_distance > max_distance:
            raise ParameterError('min_distance greater than max_distance')

        return dt.date.today() + dt.timedelta(days=random.randrange(min_distance, max_distance))

    def past_date(self, min_distance=0, max_distance=365):
        """Random date between today and today + one year - one day."""
        if min_distance < 0:
            raise ParameterError('min_distance less than 0')

        if min_distance > max_distance:
            raise ParameterError('min_distance greater than max_distance')

        return dt.date.today() - dt.timedelta(days=random.randrange(min_distance, max_distance))

    def datetime_between(self, min_datetime, max_datetime):
        """Random datetime between a min datetime and a max datetime."""
        if min_datetime > max_datetime:
            raise ParameterError('min_datetime greater than max_datetime')

        delta = max_datetime - min_datetime
        int_delta = (delta.days * 24 * 60 * 60) + delta.seconds
        random_second = self.int(0, int_delta)

        return min_datetime + dt.timedelta(seconds=random_second)

    def future_datetime(self, min_distance=0, max_distance=1440):
        """Random date between today and today + one year - one day."""
        if min_distance < 0:
            raise ParameterError('min_distance less than 0')

        if min_distance > max_distance:
            raise ParameterError('min_distance greater than max_distance')

        now = dt.datetime.utcnow().replace(tzinfo=utc)
        delta = dt.timedelta(minutes=random.randrange(min_distance, max_distance))

        return now + delta

    def past_datetime(self, min_distance=0, max_distance=1440):
        """Random date between today and today + one year - one day."""
        if min_distance < 0:
            raise ParameterError('min_distance less than 0')

        if min_distance > max_distance:
            raise ParameterError('min_distance greater than max_distance')

        now = dt.datetime.utcnow().replace(tzinfo=utc)
        delta = dt.timedelta(minutes=random.randrange(min_distance, max_distance))

        return now - delta

    def date(self, begin=-365, end=365):
        """Random date between today - one year and today + one year."""
        if begin > end:
            raise ParameterError('begin greater than end')

        return dt.date.today() - dt.timedelta(random.randrange(begin, end))

    def datetime(self, begin=-1440, end=1440):
        """Random date between today - one year and today + one year."""
        if begin > end:
            raise ParameterError('begin greater than end')

        return dt.datetime.utcnow().replace(tzinfo=utc) - dt.timedelta(minutes=random.randrange(begin, end))

    def time(self):
        """Random time"""
        return dt.time(
            self.int(0, 23),
            self.int(0, 59),
            self.int(0, 59),
            self.int(0, 999999),
        )



from django.conf.urls import url, patterns

urlpatterns = patterns('',
    url(r'^add_sample/(?P<app_label>[a-z0-9_]+)/(?P<model>[a-z0-9_]+)/$', 'sampledatahelper.admin.views.add_sample', 'sampledatahelper-add-sample'),
)



import random
import string
from ..exceptions import ParameterError

from django.contrib.webdesign import lorem_ipsum


class TextMixin(object):
    def word(self):
        """Random text with 1 word."""
        return lorem_ipsum.words(1, common=False)

    def words(self, min_words=1, max_words=5):
        """Random text with 1 word."""

        if min_words > max_words:
            raise ParameterError('min_words greater than max_words')

        words = random.randint(min_words, max_words)
        return lorem_ipsum.words(words, common=False)

    def char(self):
        """Random character."""
        return random.choice(string.ascii_letters)

    def chars(self, min_chars=1, max_chars=5):
        """Random text with 1 word."""

        if min_chars > max_chars:
            raise ParameterError('min_chars greater than max_chars')

        chars = random.randint(min_chars, max_chars)
        result = ''
        for _ in range(chars):
            result += self.char()
        return result

    def email(self):
        """Random mail address."""
        username = lorem_ipsum.words(1, common=False)
        domain = lorem_ipsum.words(1, common=False)
        termination = random.choice([u'.com', u'.org', u'.net'])

        return "{0}@{1}{2}".format(username, domain, termination)

    def url(self):
        """Random url."""
        protocol = random.choice(["http", "https"])
        domain = self.word()
        termination = random.choice([u'.com', u'.org', u'.net'])
        path = self.word()

        return "{0}://{1}{2}/{3}".format(protocol, domain, termination, path)

    def sentence(self):
        """Random sentence with text shorter than 255 characters."""
        sentence = lorem_ipsum.sentence()
        while len(sentence) >= 255:
            sentence = lorem_ipsum.sentence()
        return sentence

    def short_sentence(self):
        """Random sentence with text shorter than 100 characters."""
        sentence = lorem_ipsum.sentence()
        while len(sentence) >= 100:
            sentence = lorem_ipsum.sentence()
        return sentence

    def long_sentence(self):
        """Random sentence with text longer than 150 characters."""
        sentence = lorem_ipsum.sentence()
        while len(sentence) <= 150:
            sentence = lorem_ipsum.sentence()
        return sentence

    def paragraph(self):
        """Random text with variable number of words, several sentences."""
        return lorem_ipsum.paragraph()

    def paragraphs(self, min_paragraphs=1, max_paragraphs=5):
        """Random text with variable number of words, several sentences."""

        if min_paragraphs > max_paragraphs:
            raise ParameterError('min_paragraphs greater than max_paragraphs')

        return "\n\n".join(lorem_ipsum.paragraphs(random.randrange(min_paragraphs, max_paragraphs+1)))

    def slug(self, min_words=5, max_words=5):
        """Random slug"""

        if min_words > max_words:
            raise ParameterError('min_words greater than max_words')

        return "-".join([self.word() for x in range(self.int(max_value=max_words, min_value=min_words))])

    def tags(self, min_tags=1, max_tags=5, tags_list=None):
        if min_tags > max_tags:
            raise ParameterError('min_tags greater than max_tags')

        tags = []
        for i in range(random.randrange(min_tags, max_tags+1)):
            if tags_list:
                tags.append(tags_list[random.randrange(0, len(tags_list))])
            else:
                tags.append(self.word())
        return ','.join(tags)



from .text_mixin import TextMixin
from .number_mixin import NumberMixin
from .time_mixin import TimeMixin
from .localized_mixin import LocalizedMixin
from .image_mixin import ImageMixin
from .other_mixin import OtherMixin



from django.db import models
from django.db.models.fields import FieldDoesNotExist

from .helper import SampleDataHelper
from .exceptions import ParameterError
from .register import register

# import the logging library
import logging

# Get an instance of a logger
logger = logging.getLogger('sampledatahelper')


class ModelDataHelper(object):
    def __init__(self, seed=None):
        self.sd = SampleDataHelper(seed)

    def __get_instance_fields(self, instance):
        fields = []
        for field_name in instance._meta.get_all_field_names():
            try:
                fields.append((field_name, instance._meta.get_field(field_name)))
            except FieldDoesNotExist:
                pass
        return fields

    def fill_model(self, model, number, *args, **kwargs):
        if number <= 0:
            raise ParameterError('number must be greater than 0')

        if not issubclass(model, models.Model):
            raise ParameterError('model must be a django model subclass')

        logger.debug("Filling model %s.%s with %d instances" % (model._meta.app_label, model.__name__, number))

        for x in range(number):
            instance = model()
            logger.debug("Filling instance %d" % (x))
            self.fill_model_instance(instance, *args, **kwargs)
            instance.save()

    def fill_model_instance(self, instance, *args, **kwargs):
        if not isinstance(instance, models.Model):
            raise ParameterError('instance must be a django model instance')

        for field_name, field_obj in self.__get_instance_fields(instance):
            if field_name not in kwargs:
                handler = register.get_handler(field_obj)
                if handler:
                    value = handler.generate()
                    setattr(instance, field_name, value)

        for field in args:
            if hasattr(field[1], '__call__'):
                value = field[1](instance, self.sd)
            else:
                value = field[1]
            setattr(instance, field[0], value)

        for field_name in kwargs.keys():
            if hasattr(kwargs[field_name], '__call__'):
                value = kwargs[field_name](instance, self.sd)
            else:
                value = kwargs[field_name]
            setattr(instance, field_name, value)



SECRET_KEY = "123"

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3', # Add 'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': ':memory:',                      # Or path to database file if using sqlite3.
        'USER': '',
        'PASSWORD': '',
        'HOST': '',                      # Empty for localhost through domain sockets or '127.0.0.1' for localhost through TCP.
        'PORT': '',                      # Set to empty string for default.
    }
}

INSTALLED_APPS = [
    'django.contrib.sessions',
    'django.contrib.contenttypes',
    'django.contrib.auth',
    'sampledatahelper',
    'tests'
]

ALLOWED_HOSTS = ['testserver',]

BACKEND_SESSION_KEY = "test"

USE_TZ = True

SAMPLEDATAHELPER_MODELS = [
    {
        'model': 'tests.TestRelatedModel',
        'number': 10,
    },
    {
        'model': 'tests.TestModel',
        'number': 5,
        'fields_overwrite': [
            ('small_integer', lambda _, sd: sd.int(1, 10)),
            ('integer', lambda instance, _: instance.small_integer * 2),
            ('positive_integer', 8),
        ]
    }
]



import sys
import random

from ..exceptions import ParameterError


class NumberMixin(object):
    def int(self, min_value=0, max_value=sys.maxsize):
        """Random number from 0 or min_value to max_value - 1 or sys.maxint - 1."""

        if min_value > max_value:
            raise ParameterError('min_value greater than max_value')

        return random.randrange(min_value, max_value + 1)

    def number(self, ndigits):
        """Random number with the given number of digits as maximum."""

        if ndigits < 1:
            raise ParameterError('ndigits is less than 1')

        return random.randrange(0, 10 ** ndigits)

    def digits(self, ndigits):
        """Random number with exactly the given number of digits."""
        if ndigits < 1:
            raise ParameterError('ndigits is less than 1')

        return random.randrange(10 ** (ndigits-1), 10 ** ndigits)

    def float(self, min_value, max_value):
        """Random float from min to max"""

        if min_value > max_value:
            raise ParameterError('min_value greater than max_value')

        return (max_value - (random.random() * (max_value - min_value)))

    def number_string(self, ndigits):
        """Random number from 0 to ndigits, in string format, filled by 0s on the left."""

        if ndigits < 0:
            raise ParameterError('ndigits is less than 0')

        return u''.join(random.choice(u'0123456789') for i in range(ndigits))



from django.db.models.loading import get_model
from sampledatahelper.model_helper import ModelDataHelper

def add_sample(request, app_label, model):
    mdh = ModelDataHelper(seed=12345678901)
    if filter(lambda x: x['model'] == "{}.{}".format(app_label, model), settings.SAMPLEDATAHELPER_MODELS):
        model = get_model(app_label, model)
        instance = model()
        mdh.fill_model_instance(instance)
        instance.save()



import math
from PIL import Image, ImageDraw


class ImgSimple(object):
    def generate(self, sd, width, height):
        im = Image.new("RGBA", (width, height))
        bg = Image.new(
            "RGBA",
            (width, height),
            (1 + sd.int(0, 254), 1 + sd.int(0, 254), 1 + sd.int(0, 254), 255)
        )
        draw = ImageDraw.Draw(im)
        draw.text(
            (sd.int(width / 2), sd.int(height / 2)),
            sd.word(),
            #font=sd.ttf_font,
            fill="rgb(%d,%d,%d)" % (1 + sd.int(0, 254), 1 + sd.int(0, 254), 1 + sd.int(0, 254))
        )
        im = im.rotate(-sd.int(90))
        bg.paste(im, (0, 0), im)
        return bg


class ImgPlasma(object):
    def generate(self, sd, width, height):
        self.sd = sd
        im = Image.new("L", (width, height))
        bg = Image.new(
            "RGBA",
            (width, height),
            (1 + sd.int(0, 254), 1 + sd.int(0, 254), 1 + sd.int(0, 254), 255)
        )
        # image size
        self.roughness = sd.int(2, 5)

        self.im = im
        self.im.putpixel((0, 0), sd.int(0, 254))
        self.im.putpixel((width-1, 0), sd.int(0, 254))
        self.im.putpixel((width-1, height - 1), sd.int(0, 254))
        self.im.putpixel((0, height - 1), sd.int(0, 254))
        self.subdivide(0, 0, width - 1, height - 1)
        self.im = self.im.convert("RGBA")
        return Image.blend(self.im, bg, 0.5)

    def adjust(self, xa, ya, x, y, xb, yb):
        if(self.im.getpixel((x, y)) == 0):
            d = math.fabs(xa - xb) + math.fabs(ya - yb)
            v = (
                (
                    self.im.getpixel((xa, ya))
                    + self.im.getpixel((xb, yb))
                )
                / 2.0
                + (self.sd.float(0, 1) - 0.5)
                * d
                * self.roughness
            )
            c = int(math.fabs(v) % 256)
            self.im.putpixel((x, y), c)

    def subdivide(self, x1, y1, x2, y2):
        if(not((x2-x1 < 2.0) and (y2-y1 < 2.0))):
            x = int((x1 + x2) / 2.0)
            y = int((y1 + y2) / 2.0)
            self.adjust(x1, y1, x, y1, x2, y1)
            self.adjust(x2, y1, x2, y, x2, y2)
            self.adjust(x1, y2, x, y2, x2, y2)
            self.adjust(x1, y1, x1, y, x1, y2)
            if(self.im.getpixel((x, y)) == 0):
                v = int(
                    (
                        self.im.getpixel((x1, y1))
                        + self.im.getpixel((x2, y1))
                        + self.im.getpixel((x2, y2))
                        + self.im.getpixel((x1, y2))
                    )
                    / 4.0
                )
                self.im.putpixel((x, y), v)

            self.subdivide(x1, y1, x, y)
            self.subdivide(x, y1, x2, y)
            self.subdivide(x, y, x2, y2)
            self.subdivide(x1, y, x, y2)


class ImgMandelbrot(object):
    def generate(self, sd, width, height):
        im = Image.new("RGBA", (width, height))
        mtx = im.load()

        xa = sd.float(-2, 0)
        xb = sd.float(0, 2)
        ya = sd.float(-2, 0)
        yb = sd.float(0, 2)
        maxIt = 20  # max iterations allowed

        #optimizations
        lutx = [j * (xb-xa) / (width - 1) + xa for j in range(width)]

        for y in range(height):
            cy = y * (yb - ya) / (height - 1) + ya
            for x in range(width):
                c = complex(lutx[x], cy)
                z = 0
                for i in range(maxIt):
                    if abs(z) > 2.0:
                        break
                    z = z * z + c
                r = i % 4 * 64
                g = i % 8 * 32
                b = i % 16 * 16
                mtx[x, y] = r, g, b
        return im


class ImgIFS(object):
    def generate(self, sd, width, height):
        im = Image.new("RGBA", (width, height))
        bg = Image.new(
            "RGBA",
            (width, height),
            (1 + sd.int(0, 254), 1 + sd.int(0, 254), 1 + sd.int(0, 254), 255)
        )
        mats = []
        ### Fractint IFS definition of Fern
        mats.append([[0.0, 0.0, 0.0, 0.16, 0.0, 0.0, 0.01],
                    [0.85, 0.04, -0.04, 0.85, 0.0, 1.6, 0.85],
                    [0.2, -0.26, 0.23, 0.22, 0.0, 1.6, 0.07],
                    [-0.15, 0.28, 0.26, 0.24, 0.0, 0.44, 0.07]])
        ### Fractint IFS definition of Dragon
        mats.append([[0.824074, 0.281482, -0.212346,  0.864198, -1.882290, -0.110607, 0.787473],
                    [0.088272, 0.520988, -0.463889, -0.377778,  0.785360,  8.095795, 0.212527]])
        ### Levy C curve
        mats.append([[0.5, -0.5, 0.5, 0.5, 0.0, 0.0, 0.5],
                    [0.5, 0.5, -0.5, 0.5, 0.5, 0.5, 0.5]])

        # Levy Dragon
        mats.append([[0.5, -0.5, 0.5, 0.5, 0.0, 0.0, 0.5],
                    [-0.5, -0.5, 0.5, -0.5, 1.0, 0.0, 0.5]])

        # Piramid
        mats.append([[0.5, 0.0, 0.0, 0.5, 0.0, 0.0, 0.33],
                    [0.5, 0.0, 0.0, 0.5, 1.0, 0.0, 0.33],
                    [0.5, 0.0, 0.0, 0.5, 0.6, 0.8, 0.33]])

        mat = sd.choice(mats)

        # image size
        imgx = width
        imgy = height

        m = len(mat)
        # find the xmin, xmax, ymin, ymax
        x = mat[0][4]
        y = mat[0][5]
        #
        xa = x
        xb = x
        ya = y
        yb = y
        #
        for k in range(imgx * imgy):
            p = sd.float(0, 1)
            psum = 0.0
            for i in range(m):
                psum += mat[i][6]
                if p <= psum:
                    break
            x0 = x * mat[i][0] + y * mat[i][1] + mat[i][4]
            y = x * mat[i][2] + y * mat[i][3] + mat[i][5]
            x = x0
            #
            if x < xa:
                xa = x
            if x > xb:
                xb = x
            if y < ya:
                ya = y
            if y > yb:
                yb = y

        # drawing
        x = 0.0
        y = 0.0
        colour = (sd.int(0, 254), sd.int(0, 254), sd.int(0, 254))
        for k in range(imgx * imgy):
            p = sd.float(0, 1)
            psum = 0.0
            for i in range(m):
                psum += mat[i][6]
                if p <= psum:
                    break
            x0 = x * mat[i][0] + y * mat[i][1] + mat[i][4]
            y = x * mat[i][2] + y * mat[i][3] + mat[i][5]
            x = x0
            jx = int((x - xa) / (xb - xa) * (imgx - 1))
            jy = (imgy - 1) - int((y - ya) / (yb - ya) * (imgy - 1))
            if jx >= 0 and jx < width and jy >= 0 and jy < height:
                im.putpixel((jx, jy), colour)
        im = im.rotate(sd.int(360))
        bg.paste(im, (0, 0), im)
        return bg



from django.db import models
from sampledatahelper.helper import SampleDataHelper
from sampledatahelper import handlers


class Register(object):
    fields = {}
    ignored_fields = []
    _instance = None
    sd = SampleDataHelper()

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(Register, cls).__new__(cls, *args, **kwargs)
        return cls._instance

    def register(self, field_class, handler_class):
        self.fields[field_class] = handler_class

    def ignore(self, field_class):
        self.ignored_fields.append(field_class)

    def get_handler(self, field_instance):
        if field_instance.__class__ in self.ignored_fields:
            return None
        handler = self.fields.get(field_instance.__class__, None)
        if handler:
            return handler(self.sd, field_instance)
        return None


register = Register()
register.register(models.CharField, handlers.CharHandler)
register.register(models.BigIntegerField, handlers.BigIntegerHandler)
register.register(models.CharField, handlers.CharHandler)
register.register(models.SlugField, handlers.SlugHandler)
register.register(models.EmailField, handlers.EmailHandler)
register.register(models.URLField, handlers.URLHandler)
register.register(models.TextField, handlers.TextHandler)
register.register(models.IntegerField, handlers.IntegerHandler)
register.register(models.SmallIntegerField, handlers.SmallIntegerHandler)
register.register(models.PositiveIntegerField, handlers.PositiveIntegerHandler)
register.register(models.PositiveSmallIntegerField, handlers.PositiveSmallIntegerHandler)
register.register(models.BigIntegerField, handlers.BigIntegerHandler)
register.register(models.FloatField, handlers.FloatHandler)
register.register(models.BooleanField, handlers.BooleanHandler)
register.register(models.NullBooleanField, handlers.NullBooleanHandler)
register.register(models.CommaSeparatedIntegerField, handlers.CommaSeparatedIntegerHandler)
register.register(models.DecimalField, handlers.DecimalHandler)
register.register(models.DateField, handlers.DateHandler)
register.register(models.DateTimeField, handlers.DateTimeHandler)
register.register(models.TimeField, handlers.TimeHandler)
register.register(models.FileField, handlers.FileHandler)
register.register(models.FilePathField, handlers.FilePathHandler)
register.register(models.ImageField, handlers.ImageHandler)
register.register(models.IPAddressField, handlers.IPAddressHandler)
register.register(models.GenericIPAddressField, handlers.GenericIPAddressHandler)
register.register(models.ForeignKey, handlers.ForeignKeyHandler)
register.register(models.OneToOneField, handlers.OneToOneHandler)
register.ignore(models.ManyToManyField)
register.ignore(models.AutoField)



import random
import os
import io

from django.core.files.base import ContentFile

from ..exceptions import ParameterError, NotChoicesFound


class OtherMixin(object):
    def boolean(self):
        return random.randrange(0, 2) == 0

    def nullboolean(self):
        return random.choice([None, True, False])

    def choice(self, choices):
        if not isinstance(choices, list) and not isinstance(choices, tuple):
            raise ParameterError('choices must be a list or a tuple')

        if choices == []:
            raise ParameterError('choices can\'t be a empty list')

        return random.choice(choices)

    def choices_key(self, choices):
        if not isinstance(choices, list) and not isinstance(choices, tuple):
            raise ParameterError('choices must be a list or a tuple')

        try:
            return self.choice(choices)[0]
        except (TypeError, ValueError, ParameterError):
            raise ParameterError('choices must be a valid django choices list')

    def db_object(self, model, raise_not_choices=True):
        if model.objects.all().count() > 0:
            return self.db_object_from_queryset(model.objects.all())

        if raise_not_choices:
            raise NotChoicesFound('Emtpy queryset')

        return None

    def db_object_from_queryset(self, queryset, raise_not_choices=True):
        count = queryset.all().count()
        if count > 0:
            return queryset.all()[self.int(max_value=count-1)]

        if raise_not_choices:
            raise NotChoicesFound('Emtpy queryset')

        return None

    def ipv4(self):
        return "{0}.{1}.{2}.{3}".format(
            self.int(0, 255),
            self.int(0, 255),
            self.int(0, 255),
            self.int(0, 255),
        )

    def ipv6(self):
        return "{0}:{1}:{2}:{3}:{4}:{5}:{6}:{7}".format(
            self.hex_chars(1, 4),
            self.hex_chars(1, 4),
            self.hex_chars(1, 4),
            self.hex_chars(1, 4),
            self.hex_chars(1, 4),
            self.hex_chars(1, 4),
            self.hex_chars(1, 4),
            self.hex_chars(1, 4)
        )

    def mac_address(self):
        return "{0}:{1}:{2}:{3}:{4}:{5}".format(
            self.hex_chars(2, 2),
            self.hex_chars(2, 2),
            self.hex_chars(2, 2),
            self.hex_chars(2, 2),
            self.hex_chars(2, 2),
            self.hex_chars(2, 2),
        )

    def hex_chars(self, min_chars=1, max_chars=5):

        if min_chars > max_chars:
            raise ParameterError('min_chars greater than max_chars')

        result = ""
        chars = random.randint(min_chars, max_chars)
        for x in range(chars):
            result += self.choice(['0', '1', '2', '3', '4', '5', '6', '7',
                                   '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'])
        return result

    def path(self, absolute=None, extension='', min_levels=1, max_levels=5):
        if min_levels > max_levels:
            raise ParameterError('min_levels greater than max_levels')

        if absolute is None:
            absolute = self.boolean()

        if absolute:
            result = "/"
        else:
            result = ""

        levels = random.randint(min_levels, max_levels)
        for x in range(levels):
            result += self.word()
            if x != max_levels - 1:
                result += "/"

        result += extension

        return result

    def file_from_directory(self, directory_path, valid_extensions=['.jpg', '.bmp', '.png']):
        if not os.path.exists(directory_path):
            raise ParameterError('directory_path must be a valid path')

        list_of_images = os.listdir(directory_path)
        list_of_images = list(filter(lambda x: os.path.splitext(x)[1] in valid_extensions, list_of_images))

        if len(list_of_images) == 0:
            raise NotChoicesFound('Not valid images found in directory_path for valid_extensions')

        random_path = os.path.join(directory_path, random.choice(list_of_images))

        fd = open(random_path, 'rb')
        file_data = ContentFile(fd.read(), name=os.path.basename(random_path))
        fd.close()
        return file_data



import random

from sampledatahelper.l10n.names import Name, Surname, FullName

from ..exceptions import ParameterError


class LocalizedMixin(object):
    def state_code(self, locale):
        """Random province code."""
        if locale == "es":
            return random.choice(
                ['01', '02', '03', '04', '05', '06', '07', '08', '09', '10',
                 '11', '12', '13', '14', '15', '16', '17', '18', '19', '20',
                 '21', '22', '23', '24', '25', '26', '27', '28', '29', '30',
                 '31', '32', '33', '34', '35', '36', '37', '38', '39', '40',
                 '41', '42', '43', '44', '45', '46', '47', '48', '49', '50',
                 '51', '52', 'AD', ]
            )
        elif locale == "us":
            return random.choice(
                ['AL', 'AK', 'AZ', 'AR', 'CA', 'CO', 'CT', 'DE', 'FL', 'GA',
                 'HI', 'ID', 'IL', 'IN', 'IA', 'KS', 'KY', 'LA', 'ME', 'MD',
                 'MA', 'MI', 'MN', 'MS', 'MO', 'MT', 'NE', 'NV', 'NH', 'NJ',
                 'NM', 'NY', 'NC', 'ND', 'OH', 'OK', 'OR', 'PA', 'RI', 'SC',
                 'SD', 'TN', 'TX', 'UT', 'VT', 'VA', 'WA', 'WV', 'WI', 'WY',
                 'AS', 'DC', 'FM', 'GU', 'MH', 'MP', 'PW', 'PR', 'VI', ]
            )
        else:
            raise ParameterError("Not valid locale")

    def name(self, locale=None, number=None, as_list=False):
        return Name().generate(self, locale, number, as_list)

    def surname(self, locale=None, number=None, as_list=False):
        return Surname().generate(self, locale, number, as_list)

    def fullname(self, locale=None, as_list=False):
        return FullName().generate(self, locale, as_list)

    def phone(self, locale=None, country_code=False):
        phone = ''
        if locale == "es":
            if country_code is True:
                phone += "+34 "
            phone += random.choice(['6', '9'])
            phone += str(self.int(10000000, 99999999))
            return phone
        else:
            # Only works with implemented locales
            raise ParameterError("Not valid locale")

    def zip_code(self, locale=None):
        zip_code = ''
        if locale == "es":
            zip_code = "%05d" % self.int(1000, 52999)
            return zip_code
        else:
            # Only works with implemented locales
            raise ParameterError("Not valid locale")

    def id_card(self, locale=None):
        id_card = ''
        if locale == "es":
            id_card = "%05d" % self.int(1000, 52999)
            id_card = self.number_string(8)
            id_card_letters = "TRWAGMYFPDXBNJZSQVHLCKET"
            id_card += id_card_letters[int(id_card) % 23]
            return id_card
        else:
            # Only works with implemented locales
            raise ParameterError("Not valid locale")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import random

from django.conf import settings
from .mixins import NumberMixin, TextMixin, TimeMixin, LocalizedMixin, ImageMixin, OtherMixin


class SampleDataHelper(NumberMixin, TextMixin, TimeMixin, LocalizedMixin, ImageMixin, OtherMixin):
    def __init__(self, seed=None):
        if seed is None:
            seed = getattr(settings, 'SAMPLEDATAHELPER_SEED', None)

        if seed is not None:
            random.seed(seed)



class BaseHandler(object):
    def __init__(self, sd, instance):
        self.sd = sd
        self.instance = instance

    def generate(self):
        if self.instance.choices and len(self.instance.choices) > 0:
            return self.sd.choices_key(self.instance.choices)

        return self._generate()


class CharHandler(BaseHandler):
    def _generate(self):
        if self.instance.max_length < 10:
            return self.sd.chars(1, 10)
        elif self.instance.max_length < 50:
            return self.sd.words(1, 5)[0:self.instance.max_length]
        elif self.instance.max_length < 100:
            return self.sd.words(1, 10)[0:self.instance.max_length]
        elif self.instance.max_length < 200:
            return self.sd.short_sentence()[0:self.instance.max_length]
        else:
            return self.sd.long_sentence()[0:self.instance.max_length]


class SlugHandler(BaseHandler):
    def _generate(self):
        if self.instance.max_length < 10:
            return self.sd.chars(1, 10)
        elif self.instance.max_length < 50:
            return self.sd.slug(1, 5)[0:self.instance.max_length]
        elif self.instance.max_length < 100:
            return self.sd.slug(1, 10)[0:self.instance.max_length]
        elif self.instance.max_length < 200:
            return self.sd.slug(5, 15)[0:self.instance.max_length]
        else:
            return self.sd.slug(15, 25)[0:self.instance.max_length]


class EmailHandler(BaseHandler):
    def _generate(self):
        return self.sd.email()


class URLHandler(BaseHandler):
    def _generate(self):
        return self.sd.url()


class TextHandler(BaseHandler):
    def _generate(self):
        return self.sd.paragraphs(2, 5)


class IntegerHandler(BaseHandler):
    def _generate(self):
        return self.sd.int(-1000000, 1000000)


class SmallIntegerHandler(BaseHandler):
    def _generate(self):
        return self.sd.int(-32000, 32000)


class PositiveSmallIntegerHandler(BaseHandler):
    def _generate(self):
        return self.sd.int(0, 65000)


class PositiveIntegerHandler(BaseHandler):
    def _generate(self):
        return self.sd.int()


class BigIntegerHandler(IntegerHandler):
    pass


class DecimalHandler(IntegerHandler):
    pass


class FloatHandler(BaseHandler):
    def _generate(self):
        return self.sd.float(-1000000, 1000000)


class BooleanHandler(BaseHandler):
    def _generate(self):
        return self.sd.boolean()


class NullBooleanHandler(BaseHandler):
    def _generate(self):
        return self.sd.nullboolean()


class CommaSeparatedIntegerHandler(BaseHandler):
    def _generate(self):
        integers = []
        for x in range(self.sd.int(3, 8)):
            integers.append(str(self.sd.int(0, 1000)))

        return ",".join(integers)


class DateHandler(BaseHandler):
    def _generate(self):
        return self.sd.date()


class DateTimeHandler(BaseHandler):
    def _generate(self):
        return self.sd.datetime()


class TimeHandler(BaseHandler):
    def _generate(self):
        return self.sd.time()


class FileHandler(BaseHandler):
    def _generate(self):
        return self.sd.image(100, 100, 'random')


class FilePathHandler(BaseHandler):
    def _generate(self):
        return self.sd.path()


class ImageHandler(BaseHandler):
    def _generate(self):
        return self.sd.image(100, 100, 'random')


class IPAddressHandler(BaseHandler):
    def _generate(self):
        return self.sd.ipv4()


class GenericIPAddressHandler(BaseHandler):
    def _generate(self):
        if self.sd.boolean():
            return self.sd.ipv4()
        else:
            return self.sd.ipv6()


class ForeignKeyHandler(BaseHandler):
    def _generate(self):
        if self.instance.rel.limit_choices_to:
            queryset = self.instance.rel.to.objects.filter(**self.instance.rel.limit_choices_to)
        else:
            queryset = self.instance.rel.to.objects.all()
        return self.sd.db_object_from_queryset(queryset)


class OneToOneHandler(ForeignKeyHandler):
    def _generate(self):
        if self.instance.rel.limit_choices_to:
            queryset = self.instance.rel.to.objects.filter(
                **self.instance.rel.limit_choices_to
            ).exclude(id__in=self.instance.model.objects.all().values(self.instance.name))
        else:
            quersyet = self.instance.rel.to.objects.all()
            queryset = quersyet.exclude(id__in=self.instance.model.objects.all().values(self.instance.name))
        return self.sd.db_object_from_queryset(queryset)



from django.db import models

CHAR_CHOICES = (
    ('test1', 'Test 1'),
    ('test2', 'Test 2'),
    ('test3', 'Test 3'),
    ('test4', 'Test 4'),
    ('test5', 'Test 5'),
    ('test6', 'Test 6'),
    ('test7', 'Test 7'),
)

INTEGER_CHOICES = (
    (1, 'Test 1'),
    (2, 'Test 2'),
    (3, 'Test 3'),
    (4, 'Test 4'),
    (5, 'Test 5'),
    (6, 'Test 6'),
    (7, 'Test 7'),
)


class TestRelatedModel(models.Model):
    pass


class TestModel(models.Model):
    very_short_char = models.CharField(max_length=2)
    short_char = models.CharField(max_length=10)
    char = models.CharField(max_length=30)
    middle_char = models.CharField(max_length=75)
    long_char = models.CharField(max_length=120)
    longer_char = models.CharField(max_length=250)

    very_short_slug = models.SlugField(max_length=2)
    short_slug = models.SlugField(max_length=10)
    slug = models.SlugField(max_length=30)
    middle_slug = models.SlugField(max_length=75)
    long_slug = models.SlugField(max_length=120)
    longer_slug = models.SlugField(max_length=250)

    email = models.EmailField()
    url = models.URLField()
    text = models.TextField()

    integer = models.IntegerField()
    small_integer = models.SmallIntegerField()
    positive_integer = models.PositiveIntegerField()
    positive_small_integer = models.PositiveSmallIntegerField()
    big_integer = models.BigIntegerField()
    float = models.FloatField()
    boolean = models.BooleanField()
    null_boolean = models.NullBooleanField()
    comma_separated_integers = models.CommaSeparatedIntegerField(max_length=100)
    decimal = models.DecimalField(decimal_places=10, max_digits=20)

    date = models.DateField()
    datetime = models.DateTimeField()
    time = models.TimeField()

    file = models.FileField(upload_to="tests/tmp")
    file_path = models.FilePathField()
    image = models.ImageField(upload_to="tests/tmp")

    ip = models.IPAddressField()
    generic_ip = models.GenericIPAddressField()

    foreing_key = models.ForeignKey('tests.TestRelatedModel', related_name="test_related_1")
    one_to_one = models.OneToOneField(TestRelatedModel, related_name="test_related_2")
    many_to_many = models.ManyToManyField(TestRelatedModel, related_name="test_related_3")

    # With choices
    integer_choices = models.IntegerField(choices=INTEGER_CHOICES)
    char_choices = models.CharField(max_length=30, choices=CHAR_CHOICES)
    foreing_key_choices = models.ForeignKey('tests.TestRelatedModel',
                                            related_name="test_related_4",
                                            limit_choices_to={'id__gt': 0})
    one_to_one_key_choices = models.OneToOneField('tests.TestRelatedModel',
                                                  related_name="test_related_5",
                                                  limit_choices_to={'id__gt': 0})
